; $Id: jv_ratio.pro,v 1.6 2008/05/07 20:32:19 bmy Exp $
;-----------------------------------------------------------------------
;+
; NAME:
;        JV_RATIO
;
; PURPOSE:
;        Creates ratio (new/old) plots of JNO2 and JO3 at 
;        both the surface and 500 hPa. 
;       
; CATEGORY:
;        GEOS-CHEM Benchmarking
;
; CALLING SEQUENCE:
;        JV_RATIO, FILES, LEVELS, TAUS, VERSIONS, [, Keywords ]
;
; INPUTS:
;        FILES -> A 2-element vector containing the names of files
;             from the "old" and "new" GEOS-Chem model versions
;             that are to be compared. 
;
;        LEVELS -> A 4-element vector containing the level indices
;             for the GEOS-Chem surface layer and 500 hPa layer.
;             for both models (e.g. SFC_1, SFC_2, 500_1, 500_2).
;             NOTE: This is in Fortran notation (starting from 1!)
;
;        TAUS -> A 2-element vector contaning TAU values (hours GMT
;             from /1/1985) corresponding to the "old" and "new"
;             GEOS-Chem model versions.
;
;        VERSIONS -> A 2-element vector containing the version
;             numbers for the "old" and "new" GEOS-Chem model
;             versions.
;
; KEYWORD PARAMETERS:
;        /DYNRANGE -> Set this switch to create plots using the whole
;             dynamic range of the data.  Default is to restrict
;             the plot range to +/- 5%.
;
;        /PS -> Set this switch to generate PostScript output.
;
;        OUTFILENAME -> If /PS is set, will write PostScript output
;             to a file whose name is specified by this keyword.
;
; OUTPUTS:
;        None
;
; SUBROUTINES:
;        External Subroutines Required:
;        ==================================================
;        ComputeJVRatios       PlotJVRatios
;
;        External Subroutines Required:
;        ==================================================
;        CLOSE_DEVICE          COLORBAR_NDIV    (function)
;        CTM_GET_DATA          EXTRACT_FILENAME (function)       
;        GETMODELANDGRIDINFO   MULTIPANEL         
;        MYCT                  OPEN_DEVICE   
;        TVMAP  
;
; REQUIREMENTS:
;        References routines from the GAMAP package.
;
; NOTES:
;        (1) Meant to be called from BENCHMARK_1MON
;
; EXAMPLE:
;        FILES    = [ 'ctm.bpch.v7-04-10', 'ctm.bpch.v7-04-11' ]
;        LEVELS   = [ 1, 1, 13, 13 ]
;        TAUS     = [ NYMD2TAU( 20010701 ), NYMD2TAU( 20010701 ) ]
;        VERSIONS = [ 'v7-04-10', 'v7-04-11' ]
; 
;        JV_RATIO, FILES, LEVELS, TAUS, VERSIONS, $
;             /PS, OUTFILENAME='myplot.ps'
;
;             ; Creates J-value ratio plots [ v7-04-11 / v7-04-10 ]
;             ; for July 2001 at the surface and 500 hPa.
;             ; The max & min of the data will be fixed at +/- 5%.
;
;        JV_RATIO, FILES, LEVELS, TAUS, VERSIONS, $
;             /DYNRANGE, /PS, OUTFILENAME='myplot.ps'
;
;             ; Same as above, but this time the full dynamic
;             ; range of the data will be displayed.
;
; MODIFICATION HISTORY:
;        bmy, 14 Aug 2002: VERSION 1.01
;                          - Adapted from Isabelle Bey's "cat_plot.pro"
;                          - you may now specify different TAU values
;        bmy, 13 Sep 2002: VERSION 1.02
;                            for FILENEW and FILEOLD via the TAU0 keyword
;        bmy, 04 Oct 2004: VERSION 1.03
;                          - Remove MIN_VALID from call to TVMAP
;        bmy, 09 Nov 2007: VERSION 1.04
;                          - Modified argument list and some 
;                            internal variable names.
;        bmy, 20 Nov 2007: VERSION 1.05
;                          - Now draw out-of-bounds triangles for
;                            the colorbar when using the "small"
;                            data ranges.  New feature of TVMAP.
;        bmy, 07 May 2008: VERSION 1.06
;                          - Now allow for comparing models on 2
;                            different vertical grids.
; 
;-
; Copyright (C) 2007-2008,
; Bob Yantosca and Philippe Le Sager, Harvard University
; This software is provided as is without any warranty whatsoever. 
; It may be freely used, copied or distributed for non-commercial 
; purposes. This copyright notice must be kept with any copy of 
; this software. If this software shall be used commercially or 
; sold as part of a larger package, please contact the author.
; Bugs and comments should be directed to bmy@io.as.harvard.edu
; or phs@io.as.harvard.edu with subject "IDL routine ratios"
;-----------------------------------------------------------------------

pro ComputeJVRatios, Size, Data_Sfc_1, Data_Sfc_2, Ratio_Sfc, $
                           Data_500_1, Data_500_2, Ratio_500

   ; Define ratio arrays
   Ratio_Sfc  = FltArr( Size[0], Size[1] )
   Ratio_500  = FltArr( Size[0], Size[1] )

   ; Compute J-value ratio at the surface -- avoid division by zero
   Ind            = Where( Data_Sfc_1 ne 0. )
   Ratio_Sfc[Ind] = Data_Sfc_2[Ind] / Data_Sfc_1[Ind]
   
   ; Compute J-value ratio at 500 hPa -- avoid division by zero
   Ind            = Where( Data_500_1 ne 0. )
   Ratio_500[Ind] = Data_500_2[Ind] / Data_500_1[Ind]

end

;------------------------------------------------------------------------------

pro PlotJVRatio, Data, Level, TracerName, GridInfo, DynRange, _EXTRA=e
   
   ;====================================================================
   ; Internal routine PLOTRATIO plots either the surface or 500 hPa
   ; ratio of tracer between old and new versions (bmy, 11/9/07)
   ;====================================================================

   ; Plot title
   if ( Level gt 1 )                                           $
      then Title = 'J-Value Ratio @ 500 hPa for ' + TracerName $
      else Title = 'J-Value Ratio @ Surface for ' + TracerName  

   ; Number of colorbar tickmarks
   Divisions = ColorBar_NDiv( 6 )
      
   ; Don't plot the polar latitudes
   XMid      = GridInfo.XMid
   YMid      = GridInfo.YMid[ 1:GridInfo.JMX-2 ]
   Data      = Data[ *, 1:GridInfo.JMX-2 ]
    
   ; GOOD is the index of "good" data points
   Good      = Where( Data gt -9.99e30 )
   if ( Good[0] lt 0 ) then Message, 'No valid data points found!'

   ; If /DYNRANGE is set, then plot the entire dyn range of data
   ; Otherwise fix the min & max of the data to +/- 5%.
   if ( DynRange ) then begin
      MinData  = Min( Data, Max=MaxData )
      ; Comment this out -- center around 1
      ;Extreme  =  Max( [ Abs( MinData ), Abs( MaxData ) ] )
      ;MinData  = -Extreme
      ;MaxData  =  Extreme
   endif else begin
      MinData  = 0.95
      MaxData  = 1.05
      BotOut   = !MYCT.BOTTOM
      Triangle = 1
      NoGap    = 1
   endelse

   ; Plot data w/ country boundaries
   TvMap, Data, XMid, Ymid,                                   $
      /Countries,         /Coasts,         /Cbar,             $ 
      Division=Divisions, /Sample,         /Grid,             $
      Title=Title,        MinData=MinData, MaxData=MaxData,   $
      CBFormat='(f13.3)', BOR_Label=' ',   Triangle=Triangle, $
      NoGap=NoGap,        BotOut=BotOut,   _EXTRA=e

end

;------------------------------------------------------------------------------

pro JV_Ratio, Files, Levels, Taus, Versions, $
              DynRange=DynRange, PS=PS, OutFileName=OutFileName, _EXTRA=e

   ;====================================================================
   ; Initialization
   ;====================================================================
   
   ; External functions
   FORWARD_FUNCTION ColorBar_NDiv, Extract_FileName

   ; Arguments
   if ( N_Elements( Files    ) ne 2 ) then Message, 'Invalid FILES!'
   if ( N_Elements( Levels   ) ne 4 ) then Message, 'Invalid LEVELS'
   if ( N_Elements( Taus     ) ne 2 ) then Message, 'Invalid TAUS!'
   if ( N_Elements( Versions ) ne 2 ) then Message, 'Invalid VERSIONS!'

   ; Keywords
   DynRange = Keyword_Set( DynRange )
   if ( N_Elements( OutFileName ) ne 1 ) then OutFileName = 'ratios.ps'

   ; Define title for top of page
   TopTitle = 'GEOS-Chem ' + Versions[1] + $
      ' J-Values at Surface and 500 hPa!C!C' + $
      Extract_FileName( Files[1] ) + ' / ' + Extract_FileName( Files[0] )

   ; Save original color table
   TvLct, R, G, B, /Get

   ; Load Blue-White-White-Red colortable
   MyCt, /BuWhWhRd

   ;====================================================================
   ; Read data from the files
   ;====================================================================

   ; Read data from "old" file
   CTM_Get_Data, DataInfo_1, 'JV-MAP-$', $
      File=Files[0], Tau0=Taus[0], Tracer=[1,5], /Quiet

   CTM_Get_Data, DataInfo_2, 'JV-MAP-$', $
      File=Files[1], Tau0=Taus[1], Tracer=[1,5], /Quiet

   ; Error check
   if ( N_Elements( DataInfo_1 ) ne N_Elements( DataInfo_2 ) ) $
      then Message, 'Files have different numbers of tracers in them!'

   ;====================================================================
   ; Process tracers
   ;====================================================================

   ; Number of plots per page
   Rows = 2
   Cols = 2

   ; Initialize the plot device
   Open_Device, /Color, Bits=8, /Portrait, PS=PS, FileName=OutFileName
 
   ; Number of panels per page
   MultiPanel, Rows=Rows, Cols=Cols, Margin=[ 0.03, 0.03, 0.03, 0.03 ]
  
   ; Loop over tracers
   for D = 0L, N_Elements( DataInfo_1 ) - 1L do begin

      ;-----------------------------------------------------------------
      ; Error check grid, tracer name, and data block sizes 
      ;-----------------------------------------------------------------

      ; Get MODELINFO and GRIDINFO structures
      GetModelAndGridInfo, DataInfo_1[D], ModelInfo_1, GridInfo_1
      GetModelAndGridInfo, DataInfo_2[D], ModelInfo_2, GridInfo_2

      ; Make sure grids are compatible
      if ( GridInfo_1.IMX ne GridInfo_2.IMX OR $
           GridInfo_1.JMX ne GridInfo_2.JMX )  $ 
         then Message, 'Resolution mismatch!'

      ; Make sure the tracers correspond to each other
      TracerName_1 = DataInfo_1[D].TracerName
      TracerName_2 = DataInfo_2[D].TracerName
      if ( TracerName_1 ne TracerName_2 ) then Message, 'Tracer mismatch!'

      ; Get data arrays
      Data_1 = *( DataInfo_1[D].Data )
      Data_2 = *( DataInfo_2[D].Data )

      ; Get dimensions of new & old arrays
      Size_1 = Size( Data_1, /Dim )
      Size_2 = Size( Data_2, /Dim )

      ; Stop the run if the data block sizes don't agree
      if ( Size_1[0] ne Size_2[0] ) then Message, 'Longitude mismatch!'
      if ( Size_1[1] ne Size_2[1] ) then Message, 'Latitude mismatch!'

      ;-----------------------------------------------------------------
      ; Extract data arrays for surface and 500 hPa
      ;-----------------------------------------------------------------

      ; Surface data arrays
      Data_Sfc_1 = Data_1[ *, *, Levels[0]-1 ]
      Data_Sfc_2 = Data_2[ *, *, Levels[1]-1 ]

      ; 500 hPa data arrays
      Data_500_1 = Data_1[ *, *, Levels[2]-1 ]
      Data_500_2 = Data_2[ *, *, Levels[3]-1 ]

      ; We no longer need the 3-D data arrays
      UnDefine, Data_1
      UnDefine, Data_2

      ;-----------------------------------------------------------------
      ; Compute the ratios and plot the data
      ;-----------------------------------------------------------------

      ; Get the "new" / "old" ratios at surface and 500 hPa
      ComputeJVRatios, Size_1, Data_Sfc_1, Data_Sfc_2, Ratio_Sfc, $
                               Data_500_1, Data_500_2, Ratio_500

      ; Plot the Surface ratios
      PlotJVRatio, Ratio_Sfc,  Levels[0], TracerName_1, $
                   GridInfo_1, DynRange,  _EXTRA=e

      ; Plot the 500 hPa ratios
      PlotJVRatio, Ratio_500,  Levels[2], TracerName_1, $
                   GridInfo_1, DynRange,  _EXTRA=e

      ;-----------------------------------------------------------------
      ; Print top title -- only plot when there is a new page
      ;-----------------------------------------------------------------

      if ( D mod ( Rows * Cols ) eq 0 ) then begin
         XYOutS, 0.5, 1.03, TopTitle, $
            /Normal, Color=!MYCT.BLACK, CharSize=1.0, Align=0.5
      endif
 
      ; Undefine stuff
      UnDefine, Data_500_1
      UnDefine, Data_500_2
      UnDefine, Data_Sfc_1
      UnDefine, Data_Sfc_2
      UnDefine, GridInfo_1
      UnDefine, GridInfo_2
      UnDefine, ModelInfo_1
      UnDefine, ModelInfo_2
      UnDefine, Size_1
      UnDefine, Size_2

   endfor

   ;====================================================================
   ; Cleanup and quit
   ;====================================================================
Quit:

   ; Cancel previous MULTIPANEL settings
   MultiPanel, /Off

   ; Close device
   Close_Device

   ; Restore original color table
   TvLct, R, G, B

end
 
